/*
 * Copyright 2003 CodeWeavers Inc. (Ulrich Czekalla)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#define COM_NO_WINDOWS_H
#include <stdarg.h>

#include "windef.h"
#include "winbase.h"
#include "winuser.h"
#include "winerror.h"
#include "objbase.h"
#include "wine/unicode.h"
#include "wine/debug.h"
#include "dmo.h"

WINE_DEFAULT_DEBUG_CHANNEL(msdmo);

/***************************************************************
 * MoInitMediaType
 *
 * Initialize media type structure
 */
HRESULT WINAPI MoInitMediaType(DMO_MEDIA_TYPE* pmt, DWORD cbFormat)
{
    HRESULT r = S_OK;

    TRACE("%p %ld\n", pmt, cbFormat);

    if (!pmt)
        r = E_POINTER;
    else
    {
        pmt->cbFormat = cbFormat;

        if (pmt->cbFormat)
        {
            pmt->pbFormat = HeapAlloc(GetProcessHeap(), 0, pmt->cbFormat);

	    if (!pmt->pbFormat)
                r = E_OUTOFMEMORY;
        }
    }

    return r;
}


/***************************************************************
 * MoCreateMediaType
 *
 * Allocate a new media type structure
 */
HRESULT WINAPI MoCreateMediaType(DMO_MEDIA_TYPE** ppmt, DWORD cbFormat)
{
    HRESULT r = S_OK;

    TRACE("%p %ld\n", ppmt, cbFormat);

    if (!ppmt)
        r = E_POINTER;
    else
    {
        *ppmt = (DMO_MEDIA_TYPE*) HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, 
            sizeof(DMO_MEDIA_TYPE));

        if (!*ppmt)
            r = E_OUTOFMEMORY;
	else
        {
            r = MoInitMediaType(*ppmt, cbFormat);
	    if (r != S_OK)
                MoDeleteMediaType(*ppmt);
        }
    }

    return r;
}


/***************************************************************
 * MoCopyMediaType
 *
 * Copy a new media type structure
 */
HRESULT WINAPI MoCopyMediaType(DMO_MEDIA_TYPE* pmtDest, const DMO_MEDIA_TYPE* pmtSrc)
{
    HRESULT hres = S_OK;

    TRACE("%p %p\n", pmtDest, pmtSrc);

    if (!pmtDest || !pmtSrc)
        hres = E_POINTER;
    else
    {
        pmtDest->majortype = pmtSrc->majortype;
        pmtDest->subtype= pmtSrc->subtype;
        pmtDest->bFixedSizeSamples = pmtSrc->bFixedSizeSamples;
        pmtDest->bTemporalCompression = pmtSrc->bTemporalCompression;
        pmtDest->lSampleSize = pmtSrc->lSampleSize;
        pmtDest->formattype = pmtSrc->formattype;

	if (pmtSrc->pUnk)
           IUnknown_AddRef(pmtSrc->pUnk);
        pmtDest->pUnk = pmtSrc->pUnk;

	if (pmtDest->cbFormat != pmtSrc->cbFormat)
        {
            hres = MoFreeMediaType(pmtDest);
            if (S_OK != hres)
                goto lend;

            hres = MoInitMediaType(pmtDest, pmtSrc->cbFormat);
        }

        memcpy(pmtDest->pbFormat, pmtSrc->pbFormat, pmtDest->cbFormat);
    }

lend:
    return hres;
}


/***************************************************************
 * MoDuplicateMediaType
 *
 * Duplicates a media type structure
 */
HRESULT WINAPI MoDuplicateMediaType(DMO_MEDIA_TYPE** ppmtDest, 
    const DMO_MEDIA_TYPE* pmtSrc)
{
    HRESULT hres;

    TRACE("%p %p\n", ppmtDest, pmtSrc);

    hres = MoCreateMediaType(ppmtDest, pmtSrc->cbFormat);
    if (S_OK == hres)
    {
        hres = MoCopyMediaType(*ppmtDest, pmtSrc);
	if (hres != S_OK)
            MoDeleteMediaType(*ppmtDest);
    }

    return hres;
}


/***************************************************************
 * MoFreeMediaType
 *
 * Free allocated members of a media type structure
 */
HRESULT WINAPI MoFreeMediaType(DMO_MEDIA_TYPE* pmt)
{
    HRESULT r = S_OK;

    TRACE("%p\n", pmt);

    if (!pmt)
        r = E_POINTER;
    else if (pmt->pbFormat)
    {
        HeapFree(GetProcessHeap(), 0, pmt->pbFormat);
        pmt->pbFormat = NULL;
        pmt->cbFormat = 0;
    }

    return r;
}


/***************************************************************
 * MoDeleteMediaType
 *
 * Delete a media type structure
 */
HRESULT WINAPI MoDeleteMediaType(DMO_MEDIA_TYPE* pmt)
{
    HRESULT r = S_OK;

    TRACE("%p\n", pmt);

    if (!pmt)
        r = E_POINTER;
    else
    {
        MoFreeMediaType(pmt);
	HeapFree(GetProcessHeap(), 0, pmt);
    }

    return r;
}
